#include "AD5543.h"

AD5543::AD5543(byte CSpin, byte highCurrentpin, float fullScaleCurrent, LiquidCrystal& lcd, unsigned int column):
      _CSpin(CSpin),
      _highCurrentpin(highCurrentpin),
      _fullScaleCurrent(fullScaleCurrent),
      _enabled(true),
      _highcurrentmode(false),
      _AD5543SPISettings(SPISettings(50000000, MSBFIRST, SPI_MODE0)),
      _scaler(1.0),
      _lcd(lcd),
      _column(column) {};

void AD5543::begin(float current) {
  // Configure Pins.
  pinMode(_CSpin, OUTPUT);
  pinMode(_highCurrentpin, OUTPUT);

  // Set initial pin states.
  digitalWrite(_CSpin, HIGH);
  digitalWrite(_highCurrentpin, LOW);

  SetCurrent(current);
  UpdateCurrent();
}

void AD5543::SendCommand(std::vector<byte> &data) {
  // Configure the SPI Port for this chip.
  SPI.beginTransaction(_AD5543SPISettings);
  
  // Activate CS pin.
  digitalWrite(_CSpin, LOW);
  
  // Write the data out.
  for (std::vector<byte>::iterator i=data.begin(); i != data.end(); ++i) {
    // For this we will ignore the returned data.
    SPI.transfer(*i);
  }
  
  // Deactivate CS pin.
  digitalWrite(_CSpin, HIGH);
  SPI.endTransaction();
}

float AD5543::GetCurrent(void) {
  return _current;
}

void AD5543::SetHighCurrent (void) {
  digitalWrite(_highCurrentpin, HIGH);
  _highcurrentmode = true;
}

void AD5543::SetLowCurrent (void) {
  digitalWrite(_highCurrentpin, LOW);
  _highcurrentmode = false;
}

boolean AD5543::GetCurrentMode (void) {
  return _highcurrentmode;
}

void AD5543::SetCurrent(float current) {
  _setpointCurrent = current; // _scaler can be changed to superimpose signals.
  if (_enabled) {
    _current = _setpointCurrent;
    UpdateCurrent();
  }
}

void AD5543::SetScaler(float scaler) {
  _scaler = scaler;
  if (_enabled) {
    UpdateCurrent();
  }
}

void AD5543::Enable(void) {
  _current = _setpointCurrent;
  UpdateCurrent();
}

void AD5543::Disable(void) {
  _current = 0;
  UpdateCurrent();
}

void AD5543::UpdateCurrent(void) {
  std::vector<byte> data;

  float scaledCurrent = _current * _scaler;

  unsigned int output_code;
  if (_highcurrentmode) output_code = (scaledCurrent/(_fullScaleCurrent*10))*0xFFFF;
  else output_code = (scaledCurrent/_fullScaleCurrent)*0xFFFF;
  
  data.push_back((output_code >> 8) & 0xFF); // High 8-bits of output code.
  data.push_back(output_code & 0xFF); // Low 8-bits of output code.
  SendCommand(data);
  
  _lcd.setCursor(_column, 1);
  if (scaledCurrent < 10) _lcd.print(String(scaledCurrent, 2) + "mA");
  else _lcd.print(String(scaledCurrent, 1) + "mA");
}
