#include "AD9838.h"


AD9838::AD9838(float MCLK, byte CSpin, byte RSTpin, byte PHASEpin, byte FRQpin, byte SIGNpin, LiquidCrystal& lcd, unsigned int column):
      _MCLK(MCLK),
      _CSpin(CSpin),
      _RSTpin(RSTpin),
      _PHASEpin(PHASEpin),
      _FRQpin(FRQpin),
      _SIGNpin(SIGNpin),
      _AD9838SPISettings(SPISettings(1000000, MSBFIRST, SPI_MODE2)),
      _lcd(lcd),
      _column(column) {};

void AD9838::begin(float value) {
  // Configure Pins.
  pinMode(_CSpin, OUTPUT);
  pinMode(_RSTpin, OUTPUT);
  pinMode(_PHASEpin, OUTPUT);
  pinMode(_FRQpin, OUTPUT);
  pinMode(_SIGNpin, INPUT);

  // Set initial pin states.
  digitalWrite(_CSpin, HIGH);
  digitalWrite(_RSTpin, LOW);
  digitalWrite(_PHASEpin, LOW);
  digitalWrite(_FRQpin, LOW);

  // Start the DDS with the default frequency.
  SetFreq(value);
}

void AD9838::SendCommand(std::vector<byte> &data) {
  // Configure the SPI Port for this chip.
  SPI.beginTransaction(_AD9838SPISettings);
  
  // Activate CS pin.
  digitalWrite(_CSpin, LOW);
  
  // Write the data out.
  for (std::vector<byte>::iterator i=data.begin(); i != data.end(); ++i) {
    // For this we will ignore the returned data.
    SPI.transfer(*i);
  }
  
  // Deactivate CS pin.
  digitalWrite(_CSpin, HIGH);
  SPI.endTransaction();
}

float AD9838::GetFreq(void) {
  return _Freq;
}

float AD9838::GetMCLK(void) {
  return _MCLK;
}

byte DDSout1, DDSout2, DDSout3, DDSout4;

void CompDDSOut (float freq) {
  float DDSBitRes = pow(2, 28);
  float ClckFreq = 2684354.5;
  int P1 = freq * DDSBitRes / ClckFreq;
  int Q1 = P1 / 16384;
  int R1 = P1 - 16384 * Q1;
  int Q2 = Q1 / 256;
  int R2 = Q1 - 256 * Q2;
  int Q3 = R1 / 256;
  int R3 = R1 - 256 * Q3;
  DDSout1 = R3;
  DDSout2 = Q3 + 64;
  DDSout3 = R2;
  DDSout4 = Q2 + 64;
}

void AD9838::SetFreq(float value) {
  std::vector<byte> data;
  CompDDSOut(value);
  data.push_back(0x23);
  data.push_back(0x00);
  data.push_back(DDSout2);
  data.push_back(DDSout1);
  data.push_back(DDSout4);
  data.push_back(DDSout3);
  data.push_back(0xC0);
  data.push_back(0x00);
  data.push_back(0x22);
  data.push_back(0x20);
  SendCommand(data);
  _lcd.setCursor(_column, 1);
  _lcd.print("             ");
  _lcd.setCursor(_column, 1);
  _lcd.print(String(value, 2) + "Hz");
}

void AD9838::SetMode(boolean value) {
  // Set to sine or triangle wave.
  std::vector<byte> data;
  data.push_back(0b00100000); // 1st byte of control word.
  data.push_back((byte)value<<1); // Mode 1 is triangle wave, Mode 0 is sine wave.

  SendCommand(data);
}

void AD9838::Synchronize(void) {
  digitalWrite(_RSTpin, HIGH);
  digitalWrite(_RSTpin, LOW);
}
